# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PublicKey',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('key', models.TextField(help_text="The user's RSA public key")),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='public_keys', on_delete=models.CASCADE)),
            ],
        ),
    ]
